@@ -43,6 +43,7 @@ function enable_drag() {
         drop: function( event, ui ) {
             field = $.trim($(ui.draggable).text());
             name = $.trim($(ui.draggable).children().data('name'));
+            label = $.trim($(ui.draggable).children().data('label'));
             path_verbose = $.trim($(ui.draggable).children().data('path_verbose'));
             path = $.trim($(ui.draggable).children().data('path'));
             
@@ -55,24 +56,48 @@ function enable_drag() {
             row_html = '<tr><td><span style="cursor: move;" class="ui-icon ui-icon-arrowthick-2-n-s"></span></td>';
             row_html += '<td><input id="id_displayfield_set-'+i+'-path_verbose" name="displayfield_set-'+i+'-path_verbose" readonly="readonly" type="text" value="' + path_verbose + '"/></td>';
             row_html += '<td><input id="id_displayfield_set-'+i+'-field_verbose" name="displayfield_set-'+i+'-field_verbose" readonly="readonly" type="text" value="' + field + '"/><input id="id_displayfield_set-'+i+'-path" name="displayfield_set-'+i+'-path" type="hidden" value="' + path + '"/></td>';
-            row_html += '<td><input id="id_displayfield_set-'+i+'-field" name="displayfield_set-'+i+'-field" type="hidden" value="' + name + '"/>'
+            row_html += '<td><input id="id_displayfield_set-'+i+'-field" name="displayfield_set-'+i+'-field" type="hidden" value="' + label + '"/>'
             row_html += '<input id="id_displayfield_set-'+i+'-name" name="displayfield_set-'+i+'-name" type="text" value="' + name + '"/></td>';
             row_html += '<td><input type="text" name="displayfield_set-'+i+'-sort" class="small_input" id="id_displayfield_set-'+i+'-sort">';
             row_html += '<input type="checkbox" name="displayfield_set-'+i+'-sort_reverse" id="id_displayfield_set-'+i+'-sort_reverse"></td>';
             row_html += '<td><input type="text" name="displayfield_set-'+i+'-width" class="small_input" value="15" id="id_displayfield_set-'+i+'-width"></td>';
-            row_html += '<td onclick="aggregate_tip(event)"><select id="id_displayfield_set-'+i+'-aggregate" name="displayfield_set-'+i+'-aggregate"><option selected="selected" value="">---------</option><option value="Sum">Sum</option><option value="Count">Count</option><option value="Ave">Ave</option><option value="Max">Max</option><option value="Min">Min</option></select></td>';
+            row_html += '<td onclick="aggregate_tip(event)"><select id="id_displayfield_set-'+i+'-aggregate" name="displayfield_set-'+i+'-aggregate"><option selected="selected" value="">---------</option><option value="Sum">Sum</option><option value="Count">Count</option><option value="Avg">Avg</option><option value="Max">Max</option><option value="Min">Min</option></select></td>';
+            row_html += '<td><input type="checkbox" name="displayfield_set-'+i+'-total" id="id_displayfield_set-'+i+'-total"></td>';
+            row_html += '<td><input type="checkbox" name="displayfield_set-'+i+'-group" id="id_displayfield_set-'+i+'-group"></td>';
             row_html += '<td><input type="checkbox" name="displayfield_set-'+i+'-DELETE" id="id_displayfield_set-'+i+'-DELETE">';
-            row_html += '<span class="hide_me"><input type="text" name="displayfield_set-'+i+'-position" value="999" id="id_displayfield_set-'+i+'-position"></span></td>';
+            row_html += '<span class="hide_me"><input type="text" name="displayfield_set-'+i+'-position" value="'+total_forms.val()+'" id="id_displayfield_set-'+i+'-position"></span></td>';
             row_html += '</tr>';
             $('#field_list_table > tbody:last').append(row_html);
         }
     });
-    $( "#field_filter_droppable" ).droppable({
+
+
+    $("span.button[data-choices='true']").mousedown(function() {
+        $.get('/report_builder/ajax_get_choices/', {
+            'path_verbose': $(this).data('path_verbose'),
+            'label': $(this).data('label'),
+            'root_model': $(this).data('root_model'),
+            },
+            function(data) {
+                $("span.button[data-choices='true']").data('choices', data);
+            }
+        );
+    });
+
+
+    $("#field_filter_droppable" ).droppable({
         drop: function( event, ui ) {
             field = $.trim($(ui.draggable).text());
             name = $.trim($(ui.draggable).children().data('name'));
+            label = $.trim($(ui.draggable).children().data('label'));
             path_verbose = $.trim($(ui.draggable).children().data('path_verbose'));
             path = $.trim($(ui.draggable).children().data('path'));
+            choices = $.trim($(ui.draggable).children().data('choices'));
+            filter_field_pk = $.trim($(ui.draggable).children().data('pk'));
+
+            if (field.match(/\[property\]/)) {
+                property_tip();
+            };
             
             if (name == '') return;
             
@@ -83,7 +108,7 @@ function enable_drag() {
             row_html = '<tr>'
             row_html += '<td><span style="cursor: move;" class="ui-icon ui-icon-arrowthick-2-n-s"></span></td>'
             row_html += '<td><input id="id_fil-'+i+'-path_verbose" value="'+ path_verbose +'" readonly="readonly" type="text" name="fil-'+i+'-path_verbose" maxlength="2000"></td>'
-            row_html += '<td><input type="hidden" name="fil-'+i+'-field" value="'+ name +'" id="id_fil-'+i+'-field">'
+            row_html += '<td><input type="hidden" name="fil-'+i+'-field" value="'+ label +'" id="id_fil-'+i+'-field">'
             row_html += '<input name="fil-'+i+'-field_verbose" value="'+ field +'" readonly="readonly" maxlength="2000" type="text" id="id_fil-'+i+'-field_verbose">'
             row_html += '<input type="hidden" value="'+ path +'" name="fil-'+i+'-path" id="id_fil-'+i+'-path"></td>'
             row_html += '<td><select onchange="check_filter_type(event.target)" name="fil-'+i+'-filter_type" id="id_fil-'+i+'-filter_type">'
@@ -109,6 +134,8 @@ function enable_drag() {
 </select></td>'
             if ( field.indexOf("DateField") > 0 ) {
             	row_html += '<td><input class="datepicker" id="id_fil-'+i+'-filter_value" type="text" name="fil-'+i+'-filter_value" value="" maxlength="2000"></td>'
+            } else if (choices) {
+                row_html += '<td><select id="id_fil-'+i+'-filter_value" name="fil-'+i+'-filter_value">'+choices+'</select></td>'
             } else {
                 row_html += '<td><input id="id_fil-'+i+'-filter_value" type="text" name="fil-'+i+'-filter_value" value="" maxlength="2000"></td>'
             }
@@ -172,6 +199,11 @@ function aggregate_tip() {
     $('#tip_area').show('slow');
 }
 
+function property_tip() {
+    $('#tip_area').html("NOTE: Searching on properties can be <i>very</i> slow.  It's a good idea to add some fields to help speed up your report.")
+    $('#tip_area').show('slow');
+}
+
 $(function() {
     enable_drag();
     $( "#tabs" ).tabs();
